/*
 * Project name:
      CAN_SPI_click 2st NODE (Demonstration of mikroBUS CAN SPI 5V click board)
 * Copyright:
      (c) mikroElektronika, 2014.
 * Revision History:
      20111228:
             - initial release (DA & JK);
      20140712:
             - modified for EasyPIC v7 Fusion (PT);
 * Description:
      This is code that shows simple connection of two CAN SPI modules. This is for 1st node.
      PORTC shows send and receive message.
 * Test configuration:
     MCU:             ATmega32
                      http://www.atmel.com/Images/doc2503.pdf
     Dev. Board:      EasyAVR v7 Connectivity
                      http://www.mikroe.com/easyavr/
     Oscillator:       8 MHz
     Ext. Modules:    can spi click 5v     ac:CAN_SPI_5
                      http://www.mikroe.com/eng/products/view/814/can-spi-click-5v/
                      with MCP2551 on board:
                      http://www.microchip.com/wwwproducts/Devices.aspx?dDocName=en010405
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:
      - Place Accel Click board in the mikroBUS socket 1.
      - Set J5 jumper to 5V position.
      - Turn On LEDs for PortC SW10-7.
      - Turn On switches SW5-4,5,6 to enable SPI1 port.
      - CAN SPI 5V Click 1st Node connect with wires to CAN SPI 5V Click 2st Node (L  to L and H to H).
      - Enable J2 jumper on both Click boards.
*/


/**************************************************************************************************
* Global variables
**************************************************************************************************/
unsigned char Can_Init_Flags, Can_Send_Flags, Can_Rcv_Flags;                  // can flags
unsigned char Rx_Data_Len;                                                    // received data length in bytes
char RxTx_Data[8];                                                            // can rx/tx data buffer
char Msg_Rcvd;                                                                // reception flag
const long ID_1st = 12111, ID_2nd = 3;                                        // node IDs
long Rx_ID;

/**************************************************************************************************
* CAN SPI connections
**************************************************************************************************/
sbit CanSpi_CS            at  PORTA5_bit;
sbit CanSpi_CS_Direction  at  DDA5_bit;
sbit CanSpi_Rst           at  PORTA6_bit;
sbit CanSpi_Rst_Direction at  DDA6_bit;

/**************************************************************************************************
* MAIN PROGRAM
**************************************************************************************************/
void main() {                     // Main Program

  PORTC  = 0;                     // clear PORTC
  DDRC   = 0xFF;                  // set PORTC as output

  CanSpi_CS_Direction  = 0;
  CanSpi_Rst_Direction = 0;

  Can_Init_Flags       = 0;
  Can_Send_Flags       = 0;                               // clear flags
  Can_Rcv_Flags        = 0;

  Can_Send_Flags = _CANSPI_TX_PRIORITY_0 &                // form value to be used
                   _CANSPI_TX_XTD_FRAME &                 // with CANSPIWrite
                   _CANSPI_TX_NO_RTR_FRAME;

  Can_Init_Flags = _CANSPI_CONFIG_SAMPLE_THRICE &         // form value to be used
                   _CANSPI_CONFIG_PHSEG2_PRG_ON &         // with CANSPIInit
                   _CANSPI_CONFIG_XTD_MSG &
                   _CANSPI_CONFIG_DBL_BUFFER_ON &
                   _CANSPI_CONFIG_VALID_XTD_MSG &
                   _CANSPI_CONFIG_LINE_FILTER_OFF;

  // initialize SPI1 module
  SPI1_Init();

  // Focs = 10MHz, SJW = 1, PHSEG1 = 3, PHSEG2 = 3, PROPSEG = 1, -> N = 1+3+3+1, N = 8
  // Desired Baud rate Fbaud = 125kb/s
  // BRP = Fosc/(2*N*Fbaud) = 5
  CANSPIInitialize(1,3,3,3,1,Can_Init_Flags);                                 // initialize external CANSPI module
  CANSPISetOperationMode(_CANSPI_MODE_CONFIG,0xFF);                           // set CONFIGURATION mode
  CANSPISetMask(_CANSPI_MASK_B1,-1,_CANSPI_CONFIG_XTD_MSG);                   // set all mask1 bits to ones
  CANSPISetMask(_CANSPI_MASK_B2,-1,_CANSPI_CONFIG_XTD_MSG);                   // set all mask2 bits to ones
  CANSPISetFilter(_CANSPI_FILTER_B2_F3,ID_1st,_CANSPI_CONFIG_XTD_MSG);        // set id of filter B2_F3 to 1st node ID

  CANSPISetOperationMode(_CANSPI_MODE_NORMAL,0xFF);                           // set NORMAL mode

  while(1)
  {                                                                           // endless loop
    Msg_Rcvd = CANSPIRead(&Rx_ID , RxTx_Data , &Rx_Data_Len, &Can_Rcv_Flags); // receive message
    if ((Rx_ID == ID_1st) && Msg_Rcvd) {                                      // if message received check id
      PORTC = RxTx_Data[0];                                                   // id correct, output data at PORTC
      RxTx_Data[0]++ ;                                                        // increment received data
      CANSPIWrite(ID_2nd, RxTx_Data, 1, Can_Send_Flags);                      // send incremented data back
      }
  }
}